@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0313: Message Archive Management
 
@@ -18,10 +8,10 @@ from __future__ import annotations
 
 from typing import Any
 
-import time
+from collections.abc import Generator
 from datetime import datetime
 from datetime import timedelta
-from datetime import timezone
+from datetime import UTC
 
 import nbxmpp
 from nbxmpp.errors import is_error
@@ -32,6 +22,7 @@ from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
 from nbxmpp.structs import DiscoInfo
+from nbxmpp.structs import MAMQueryData
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import StanzaHandler
 from nbxmpp.structs import StanzaIDData
@@ -42,19 +33,14 @@ from gajim.common import app
 from gajim.common import types
 from gajim.common.const import ArchiveState
 from gajim.common.const import ClientState
-from gajim.common.const import KindConstant
 from gajim.common.const import SyncThreshold
 from gajim.common.events import ArchivingIntervalFinished
 from gajim.common.events import FeatureDiscovered
-from gajim.common.events import MamMessageReceived
 from gajim.common.events import RawMamMessageReceived
-from gajim.common.helpers import AdditionalDataDict
-from gajim.common.helpers import get_retraction_text
 from gajim.common.modules.base import BaseModule
-from gajim.common.modules.misc import parse_oob
 from gajim.common.modules.util import as_task
-from gajim.common.modules.util import check_if_message_correction
-from gajim.common.modules.util import get_eme_message
+from gajim.common.storage.archive import models as mod
+from gajim.common.util.datetime import FIRST_UTC_DATETIME
 
 
 class MAM(BaseModule):
@@ -72,10 +58,10 @@ class MAM(BaseModule):
         self.handlers = [
             StanzaHandler(name='message',
                           callback=self._set_message_archive_info,
-                          priority=41),
+                          priority=42),
             StanzaHandler(name='message',
                           callback=self._mam_message_received,
-                          priority=51),
+                          priority=41),
         ]
 
         self.available = False
@@ -131,27 +117,9 @@ class MAM(BaseModule):
         else:
             expected_archive = self._con.get_own_jid()
 
+        assert properties.mam is not None
         return properties.mam.archive.bare_match(expected_archive)
 
-    def _get_unique_id(self,
-                       properties: MessageProperties
-                       ) -> tuple[str | None, str | None]:
-        if properties.type.is_groupchat:
-            return properties.mam.id, None
-
-        if properties.is_self_message:
-            return None, properties.id
-
-        if properties.is_muc_pm:
-            return properties.mam.id, properties.id
-
-        if self._con.get_own_jid().bare_match(properties.from_):
-            # message we sent
-            return properties.mam.id, properties.id
-
-        # A message we received
-        return properties.mam.id, None
-
     @staticmethod
     def _get_stanza_id(properties: MessageProperties,
                        archive_jid: str
@@ -162,7 +130,7 @@ class MAM(BaseModule):
         return None
 
     def _set_message_archive_info(self,
-                                  _con: types.xmppClient,
+                                  _con: types.NBXMPPClient,
                                   _stanza: Message,
                                   properties: MessageProperties
                                   ) -> None:
@@ -172,6 +140,7 @@ class MAM(BaseModule):
             return
 
         if properties.type.is_groupchat:
+            assert properties.jid is not None
             archive_jid = properties.jid.bare
             timestamp = properties.timestamp
 
@@ -202,16 +171,24 @@ class MAM(BaseModule):
         if not self.is_catch_up_finished(archive_jid):
             return
 
-        app.storage.archive.set_archive_infos(
-            archive_jid,
-            last_mam_id=stanza_id.id,
-            last_muc_timestamp=timestamp)
+        if timestamp is not None:
+            timestamp = datetime.fromtimestamp(timestamp, UTC)
+
+        app.storage.archive.upsert_row(
+            mod.MAMArchiveState(
+                account_=self._account,
+                remote_jid_=JID.from_string(archive_jid),
+                to_stanza_id=stanza_id.id,
+                to_stanza_ts=timestamp,
+            )
+        )
 
     def _mam_message_received(self,
-                              _con: types.xmppClient,
+                              _con: types.NBXMPPClient,
                               stanza: Message,
                               properties: MessageProperties
                               ) -> None:
+
         if not properties.is_mam_message:
             return
 
@@ -220,6 +197,7 @@ class MAM(BaseModule):
                                   stanza=stanza,
                                   properties=properties))
 
+        assert properties.mam is not None
         if not self._from_valid_archive(stanza, properties):
             self._log.warning('Message from invalid archive %s',
                               properties.mam.archive)
@@ -230,144 +208,22 @@ class MAM(BaseModule):
         if not self._is_valid_request(properties):
             self._log.warning('Invalid MAM Message: unknown query id %s',
                               properties.mam.query_id)
-            self._log.debug(stanza)
+            self._log.warning(stanza)
             raise nbxmpp.NodeProcessed
 
-        is_groupchat = properties.type.is_groupchat
-        if is_groupchat:
-            kind = KindConstant.GC_MSG
-        else:
-            if properties.from_.bare_match(self._con.get_own_jid()):
-                kind = KindConstant.CHAT_MSG_SENT
-            else:
-                kind = KindConstant.CHAT_MSG_RECV
-
-        stanza_id, message_id = self._get_unique_id(properties)
-
-        # Search for duplicates
-        if app.storage.archive.find_stanza_id(self._account,
-                                              str(properties.mam.archive),
-                                              stanza_id,
-                                              message_id,
-                                              groupchat=is_groupchat):
-            self._log.info('Found duplicate with stanza-id: %s, '
-                           'message-id: %s', stanza_id, message_id)
+        stanza_id = properties.mam.id
+        if stanza_id is None:
+            self._log.warning('Unable to determine stanza id')
+            self._log.warning(stanza)
             raise nbxmpp.NodeProcessed
 
-        additional_data = AdditionalDataDict()
-        if properties.has_user_delay:
-            # Record it as a user timestamp
-            additional_data.set_value(
-                'gajim', 'user_timestamp', properties.user_timestamp)
-
-        parse_oob(properties, additional_data)
-
-        msgtxt = properties.body
-
-        if properties.is_encrypted:
-            additional_data['encrypted'] = properties.encrypted.additional_data
-        else:
-            if properties.eme is not None:
-                msgtxt = get_eme_message(properties.eme)
-
-        if properties.is_moderation:
-            additional_data.set_value(
-                'retracted', 'by', properties.moderation.moderator_jid)
-            additional_data.set_value(
-                'retracted', 'timestamp', properties.moderation.timestamp)
-            additional_data.set_value(
-                'retracted', 'reason', properties.moderation.reason)
-
-            msgtxt = get_retraction_text(
-                properties.moderation.moderator_jid,
-                properties.moderation.reason)
-
-        if not msgtxt:
-            # For example Chatstates, Receipts, Chatmarkers
-            self._log.debug(stanza.getProperties())
-            return
-
-        jid = properties.jid.new_as_bare()
-        if properties.is_muc_pm:
-            jid = properties.jid
-
-        if properties.is_self_message:
-            # Self messages can only be deduped with origin-id
-            if message_id is None:
-                self._log.warning('Self message without origin-id found')
-                return
-            stanza_id = message_id
-
-        occupant_id = self._get_occupant_id(properties)
-        real_jid = self._get_real_jid(properties)
-
-        displaymarking = None
-        if properties.has_security_label:
-            displaymarking = properties.security_label.displaymarking
-
-        event_attr: dict[str, Any] = {
-            'account': self._account,
-            'jid': jid,
-            'msgtxt': properties.body,
-            'properties': properties,
-            'additional_data': additional_data,
-            'unique_id': properties.id,
-            'stanza_id': stanza_id,
-            'archive_jid': properties.mam.archive,
-            'kind': kind,
-            'occupant_id': occupant_id,
-            'real_jid': real_jid,
-            'displaymarking': displaymarking
-        }
-
-        if check_if_message_correction(properties,
-                                       self._account,
-                                       properties.jid,
-                                       properties.body,
-                                       kind,
-                                       properties.mam.timestamp,
-                                       self._log):
-            return
-
-        event_attr['msg_log_id'] = app.storage.archive.insert_into_logs(
-            self._account,
-            jid,
-            properties.mam.timestamp,
-            kind,
-            message=msgtxt,
-            contact_name=properties.muc_nickname,
-            additional_data=additional_data,
-            stanza_id=stanza_id,
-            message_id=properties.id,
-            occupant_id=occupant_id,
-            real_jid=real_jid,
-        )
-
-        app.ged.raise_event(MamMessageReceived(**event_attr))
-
-    def _get_real_jid(self, properties: MessageProperties) -> JID | None:
-        if not properties.type.is_groupchat:
-            return None
-
-        if properties.muc_user is None:
-            return None
-
-        return properties.muc_user.jid
-
-    def _get_occupant_id(self, properties: MessageProperties) -> str | None:
-        if not properties.type.is_groupchat:
-            return None
-
-        if properties.occupant_id is None:
-            return None
-
-        contact = self._client.get_module('Contacts').get_contact(
-            properties.jid.bare, groupchat=True)
-        if contact.supports(Namespace.OCCUPANT_ID):
-            return properties.occupant_id
-        return None
+        if app.storage.archive.check_if_stanza_id_exists(
+            self._account, properties.remote_jid, stanza_id):
+            self._log.info('Received duplicated message from MAM: %s', stanza_id)
+            raise nbxmpp.NodeProcessed
 
     def _is_valid_request(self, properties: MessageProperties) -> bool:
+        assert properties.mam is not None
         valid_id = self._mam_query_ids.get(properties.mam.archive, None)
         return valid_id == properties.mam.query_id
 
@@ -378,11 +234,12 @@ class MAM(BaseModule):
 
     def _get_query_params(self) -> tuple[str | None, datetime | None]:
         own_jid = self._con.get_own_jid().bare
-        archive = app.storage.archive.get_archive_infos(own_jid)
+        archive = app.storage.archive.get_mam_archive_state(
+            self._account, own_jid)
 
         mam_id = None
         if archive is not None:
-            mam_id = archive.last_mam_id
+            mam_id = archive.to_stanza_id
 
         start_date = None
         if mam_id:
@@ -391,22 +248,22 @@ class MAM(BaseModule):
 
         else:
             # First Start, we request the last week
-            start_date = datetime.now(timezone.utc) - timedelta(days=7)
+            start_date = datetime.now(UTC) - timedelta(days=7)
             self._log.info('Request archive: %s, after date %s',
                            own_jid, start_date)
         return mam_id, start_date
 
     def _get_muc_query_params(self,
                               jid: JID,
-                              threshold: SyncThreshold
+                              threshold: int
                               ) -> tuple[str | None, datetime | None]:
 
-        archive = app.storage.archive.get_archive_infos(jid)
+        archive = app.storage.archive.get_mam_archive_state(self._account, jid)
         mam_id = None
         start_date = None
-        now = datetime.now(timezone.utc)
+        now = datetime.now(UTC)
 
-        if archive is None or archive.last_mam_id is None:
+        if archive is None or archive.to_stanza_id is None:
             # First join
             start_date = now - timedelta(days=1)
             self._log.info('Request archive: %s, after date %s',
@@ -415,19 +272,18 @@ class MAM(BaseModule):
         elif threshold == SyncThreshold.NO_THRESHOLD:
             # Not our first join and no threshold set
 
-            mam_id = archive.last_mam_id
+            mam_id = archive.to_stanza_id
             self._log.info('Request archive: %s, after mam-id %s',
-                           jid, archive.last_mam_id)
+                           jid, archive.to_stanza_id)
 
         else:
             # Not our first join, check how much time elapsed since our
             # last join and check against threshold
-            last_timestamp = archive.last_muc_timestamp
-            if last_timestamp is None:
+            last = archive.to_stanza_ts
+            if last is None:
                 self._log.info('No last muc timestamp found: %s', jid)
-                last_timestamp = 0
+                last = FIRST_UTC_DATETIME
 
-            last = datetime.fromtimestamp(float(last_timestamp), timezone.utc)
             if now - last > timedelta(days=threshold):
                 # To much time has elapsed since last join, apply threshold
                 start_date = now - timedelta(days=threshold)
@@ -437,14 +293,14 @@ class MAM(BaseModule):
 
             else:
                 # Request from last mam-id
-                mam_id = archive.last_mam_id
+                mam_id = archive.to_stanza_id
                 self._log.info('Request archive: %s, after mam-id %s:',
-                               jid, archive.last_mam_id)
+                               jid, archive.to_stanza_id)
 
         return mam_id, start_date
 
     @as_task
-    def request_archive_on_signin(self):
+    def request_archive_on_signin(self) -> Generator[Any, Any]:
         _task = yield  # noqa: F841
 
         own_jid = self._con.get_own_jid().bare
@@ -461,32 +317,44 @@ class MAM(BaseModule):
                 self._log.warning(result)
                 return
 
-            app.storage.archive.reset_archive_infos(result.jid)
+            self._log.warning(result)
+            self._log.warning('Reset archive state: %s', own_jid)
+            app.storage.archive.reset_mam_archive_state(
+                self._account, result.jid)
             _, start_date = self._get_query_params()
             result = yield self._execute_query(result.jid, None, start_date)
             if is_error(result):
                 self._log.warning(result)
                 return
 
+        assert isinstance(result, MAMQueryData)
         if result.rsm.last is not None:
             # <last> is not provided if the requested page was empty
             # so this means we did not get anything hence we only need
             # to update the archive info if <last> is present
-            app.storage.archive.set_archive_infos(
-                result.jid,
-                last_mam_id=result.rsm.last,
-                last_muc_timestamp=time.time())
+            app.storage.archive.upsert_row(
+                mod.MAMArchiveState(
+                    account_=self._account,
+                    remote_jid_=result.jid,
+                    to_stanza_id=result.rsm.last,
+                    to_stanza_ts=datetime.now(UTC),
+                )
+            )
 
         if start_date is not None:
             # Record the earliest timestamp we request from
             # the account archive. For the account archive we only
             # set start_date at the very first request.
-            app.storage.archive.set_archive_infos(
-                result.jid,
-                oldest_mam_timestamp=start_date.timestamp())
+            app.storage.archive.upsert_row(
+                mod.MAMArchiveState(
+                    account_=self._account,
+                    remote_jid_=result.jid,
+                    from_stanza_ts=start_date
+                )
+            )
 
     @as_task
-    def request_archive_on_muc_join(self, jid: JID):
+    def request_archive_on_muc_join(self, jid: JID) -> Generator[Any, Any]:
         _task = yield  # noqa: F841
 
         threshold = app.settings.get_group_chat_setting(self._account,
@@ -509,7 +377,10 @@ class MAM(BaseModule):
                 contact.notify('mam-sync-error', result.get_text())
                 return
 
-            app.storage.archive.reset_archive_infos(result.jid)
+            self._log.warning(result)
+            self._log.warning('Reset archive state: %s', jid)
+            app.storage.archive.reset_mam_archive_state(
+                self._account, result.jid)
             _, start_date = self._get_muc_query_params(jid, threshold)
             result = yield self._execute_query(result.jid, None, start_date)
             if is_error(result):
@@ -517,14 +388,19 @@ class MAM(BaseModule):
                 contact.notify('mam-sync-error', result.get_text())
                 return
 
+        assert isinstance(result, MAMQueryData)
         if result.rsm.last is not None:
             # <last> is not provided if the requested page was empty
             # so this means we did not get anything hence we only need
             # to update the archive info if <last> is present
-            app.storage.archive.set_archive_infos(
-                result.jid,
-                last_mam_id=result.rsm.last,
-                last_muc_timestamp=time.time())
+            app.storage.archive.upsert_row(
+                mod.MAMArchiveState(
+                    account_=self._account,
+                    remote_jid_=result.jid,
+                    to_stanza_id=result.rsm.last,
+                    to_stanza_ts=datetime.now(UTC)
+                )
+            )
 
         contact.notify('mam-sync-finished')
 
@@ -533,7 +409,7 @@ class MAM(BaseModule):
                        jid: JID,
                        mam_id: str | None,
                        start_date: datetime | None
-                       ):
+                       ) -> Generator[Any, Any]:
         _task = yield  # noqa: F841
 
         if jid in self._catch_up_finished:
@@ -550,9 +426,16 @@ class MAM(BaseModule):
 
         raise_if_error(result)
 
+        assert isinstance(result, MAMQueryData)
         while not result.complete:
-            app.storage.archive.set_archive_infos(result.jid,
-                                                  last_mam_id=result.rsm.last)
+            app.storage.archive.upsert_row(
+                mod.MAMArchiveState(
+                    account_=self._account,
+                    remote_jid_=result.jid,
+                    to_stanza_id=result.rsm.last,
+                )
+            )
+
             queryid = self._get_query_id(result.jid)
 
             result = yield self.make_query(result.jid,
@@ -609,15 +492,22 @@ class MAM(BaseModule):
         self._remove_query_id(result.jid)
 
         if start_date:
-            timestamp = start_date.timestamp()
+            timestamp = start_date
         else:
             timestamp = ArchiveState.ALL
 
+        assert isinstance(result, MAMQueryData)
         if result.complete:
             self._log.info('Request finished: %s, last mam id: %s',
                            result.jid, result.rsm.last)
-            app.storage.archive.set_archive_infos(
-                result.jid, oldest_mam_timestamp=timestamp)
+            app.storage.archive.upsert_row(
+                mod.MAMArchiveState(
+                    account_=self._account,
+                    remote_jid_=result.jid,
+                    from_stanza_ts=timestamp,
+                )
+            )
+
             app.ged.raise_event(ArchivingIntervalFinished(
                 account=self._account,
                 query_id=queryid))
