@@ -59,7 +59,7 @@ class Message(BaseModule):
         self._message_namespaces = {Namespace.ROSTERX, Namespace.IBB}
 
     def _check_if_unknown_contact(self,
-                                  _con: types.xmppClient,
+                                  _con: types.NBXMPPClient,
                                   stanza: nbxmpp.Message,
                                   properties: MessageProperties
                                   ) -> None:
@@ -77,6 +77,7 @@ class Message(BaseModule):
                                                 'ignore_unknown_contacts'):
             return
 
+        assert properties.jid is not None
         jid = properties.jid.bare
         if self._con.get_module('Roster').get_item(jid) is None:
             self._log.warning('Ignore message from unknown contact: %s', jid)
@@ -84,14 +85,15 @@ class Message(BaseModule):
             raise nbxmpp.NodeProcessed
 
     def _message_received(self,
-                          _con: types.xmppClient,
+                          _con: types.NBXMPPClient,
                           stanza: nbxmpp.Message,
                           properties: MessageProperties
                           ) -> None:
 
-        if (properties.is_pubsub or
-                properties.type.is_error or
-                properties.type.is_normal):
+        if properties.is_pubsub or properties.type.is_error:
+            return
+
+        if properties.type.is_normal and not properties.body:
             return
 
         # Check if a child of the message contains any
@@ -107,7 +109,7 @@ class Message(BaseModule):
             stanza=stanza,
             account=self._account))
 
-        if properties.is_carbon_message and properties.carbon.is_sent:
+        if properties.carbon is not None and properties.carbon.is_sent:
             # Ugly, we treat the from attr as the remote jid,
             # to make that work with sent carbons we have to do this.
             # TODO: Check where in Gajim and plugins we depend on that behavior
@@ -134,7 +136,7 @@ class Message(BaseModule):
         user_delay_ts = None
         if properties.user_timestamp is not None:
             user_delay_ts = dt.datetime.fromtimestamp(
-                properties.user_timestamp, tz=dt.timezone.utc)
+                properties.user_timestamp, tz=dt.UTC)
 
         message_id = properties.id
         if message_id is None:
@@ -197,7 +199,7 @@ class Message(BaseModule):
         oob_data = parse_oob(properties)
 
         encryption_data = None
-        if properties.is_encrypted:
+        if properties.encrypted is not None:
             encryption_data = mod.Encryption(
                 **dataclasses.asdict(properties.encrypted))
 
@@ -209,8 +211,7 @@ class Message(BaseModule):
             return
 
         securitylabel_data = None
-        if properties.has_security_label:
-            assert properties.security_label is not None
+        if properties.security_label is not None:
             displaymarking = properties.security_label.displaymarking
             if displaymarking is not None:
                 securitylabel_data = mod.SecurityLabel(
@@ -277,7 +278,7 @@ class Message(BaseModule):
                                             pk=pk))
 
     def _message_error_received(self,
-                                _con: types.xmppClient,
+                                _con: types.NBXMPPClient,
                                 stanza: nbxmpp.Message,
                                 properties: MessageProperties
                                 ) -> None:
