@@ -5,6 +5,7 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import cast
 from typing import Literal
 
 import calendar
@@ -16,13 +17,13 @@ from collections.abc import Iterator
 from collections.abc import Sequence
 from datetime import datetime
 from datetime import timedelta
-from datetime import timezone
 from pathlib import Path
 
 import sqlalchemy as sa
 from nbxmpp import JID
 from sqlalchemy import delete
 from sqlalchemy import select
+from sqlalchemy import union_all
 from sqlalchemy import update
 from sqlalchemy.dialects.sqlite import insert
 from sqlalchemy.engine import Engine
@@ -33,7 +34,6 @@ from gajim.common import app
 from gajim.common import configpaths
 from gajim.common.const import MAX_MESSAGE_CORRECTION_DELAY
 from gajim.common.events import DBMigration
-from gajim.common.helpers import get_random_string
 from gajim.common.storage.archive import migration
 from gajim.common.storage.archive.const import ChatDirection
 from gajim.common.storage.archive.const import MessageState
@@ -53,6 +53,7 @@ from gajim.common.storage.base import timeit
 from gajim.common.storage.base import VALUE_MISSING
 from gajim.common.storage.base import with_session
 from gajim.common.util.datetime import FIRST_UTC_DATETIME
+from gajim.common.util.text import get_random_string
 
 CURRENT_USER_VERSION = 11
 
@@ -556,6 +557,53 @@ class MessageArchiveStorage(AlchemyStorage):
         self._explain(session, stmt)
         return session.scalars(stmt).all()
 
+    @with_session
+    @timeit
+    def get_conversation_around_timestamp(
+        self,
+        session: Session,
+        account: str,
+        jid: JID,
+        timestamp: datetime
+    ) -> list[Message]:
+        '''
+        Loads messages around a primary key
+
+        :param account:
+            The account
+        :param jid:
+            The jid for which we request the conversation
+        :param timestamp:
+            The timestamp in the conversation
+        '''
+
+        fk_account_pk = self._get_account_pk(session, account)
+        fk_remote_pk = self._get_jid_pk(session, jid)
+
+        base_stmt = select(Message.pk).where(
+            Message.fk_remote_pk == fk_remote_pk,
+            Message.fk_account_pk == fk_account_pk,
+            Message.correction_id.is_(None),
+        )
+
+        preceding_query = base_stmt.where(
+            Message.timestamp <= timestamp,
+        ).order_by(
+            sa.desc(Message.timestamp), sa.desc(Message.pk)
+        ).limit(50).subquery()
+
+        following_query = base_stmt.where(
+            Message.timestamp > timestamp,
+        ).order_by(
+            sa.asc(Message.timestamp), sa.asc(Message.pk)
+        ).limit(50).subquery()
+
+        u_stmt = union_all(select(preceding_query), select(following_query))
+        stmt = select(Message).where(Message.pk.in_(u_stmt.scalar_subquery()))
+
+        self._explain(session, stmt)
+        return list(session.scalars(stmt).all())
+
     @with_session
     @timeit
     def get_last_conversation_row(
@@ -600,7 +648,7 @@ class MessageArchiveStorage(AlchemyStorage):
         fk_account_pk = self._get_account_pk(session, account)
         fk_remote_pk = self._get_jid_pk(session, jid)
 
-        min_time = datetime.now(timezone.utc) - timedelta(
+        min_time = datetime.now(dt.UTC) - timedelta(
             seconds=MAX_MESSAGE_CORRECTION_DELAY
         )
 
@@ -657,6 +705,43 @@ class MessageArchiveStorage(AlchemyStorage):
         return app.storage.archive.get_message_with_id(
             account, jid, reply_id)
 
+
+    @with_session
+    @timeit
+    def get_message_stanza_ids_from_occupant(
+        self,
+        session: Session,
+        account: str,
+        jid: JID,
+        occupant_id: str,
+    ) -> Sequence[str] | None:
+        """Get stanza IDs for all messages which were sent by a participant
+         with occupant_id.
+         """
+        fk_account_pk = self._get_account_pk(session, account)
+        fk_remote_pk = self._get_jid_pk(session, jid)
+        stmt = select(Occupant.pk).where(
+            Occupant.id == occupant_id,
+            Occupant.fk_remote_pk == fk_remote_pk,
+            Occupant.fk_account_pk == fk_account_pk,
+        )
+        fk_occupant_pk = session.scalar(stmt)
+        if fk_occupant_pk is None:
+            self._log.warning(
+                'Unable to get messages, unknown occupant-id: %s', occupant_id
+            )
+            return None
+
+        stmt = select(Message.stanza_id).where(
+            Message.fk_remote_pk == fk_remote_pk,
+            Message.fk_account_pk == fk_account_pk,
+            Message.fk_occupant_pk == fk_occupant_pk,
+            Message.stanza_id.isnot(None)
+        )
+        result = cast(Sequence[str] | None, session.scalars(stmt).all())
+        return result
+
+
     @with_session
     @timeit
     def search_archive(
@@ -694,7 +779,7 @@ class MessageArchiveStorage(AlchemyStorage):
         # TODO: Does not search in corrections
 
         if before is None:
-            before = datetime.now(timezone.utc)
+            before = datetime.now(dt.UTC)
 
         if after is None:
             after = FIRST_UTC_DATETIME
@@ -762,8 +847,8 @@ class MessageArchiveStorage(AlchemyStorage):
         _, days = calendar.monthrange(year, month)
         local_end = datetime.combine(datetime(year, month, days), dt.time.max)
 
-        start = local_start.astimezone(timezone.utc)
-        end = local_end.astimezone(timezone.utc)
+        start = local_start.astimezone(dt.UTC)
+        end = local_end.astimezone(dt.UTC)
 
         stmt = (
             select(
@@ -845,8 +930,8 @@ class MessageArchiveStorage(AlchemyStorage):
         fk_account_pk = self._get_account_pk(session, account)
         fk_remote_pk = self._get_jid_pk(session, jid)
 
-        start = datetime.combine(date, dt.time.min).astimezone(timezone.utc)
-        end = datetime.combine(date, dt.time.max).astimezone(timezone.utc)
+        start = datetime.combine(date, dt.time.min).astimezone(dt.UTC)
+        end = datetime.combine(date, dt.time.max).astimezone(dt.UTC)
 
         stmt = (
             select(Message.pk, Message.timestamp)
@@ -870,11 +955,11 @@ class MessageArchiveStorage(AlchemyStorage):
     @timeit
     def get_recent_muc_nicks(
         self, session: Session, account: str, jid: JID
-    ) -> list[str]:
+    ) -> Sequence[str]:
         fk_account_pk = self._get_account_pk(session, account)
         fk_remote_pk = self._get_jid_pk(session, jid)
 
-        recent = datetime.now(timezone.utc) - timedelta(days=30)
+        recent = datetime.now(dt.UTC) - timedelta(days=30)
 
         stmt = (
             select(Message.resource)
@@ -883,6 +968,7 @@ class MessageArchiveStorage(AlchemyStorage):
                 Message.fk_remote_pk == fk_remote_pk,
                 Message.fk_account_pk == fk_account_pk,
                 Message.timestamp > recent,
+                Message.direction == ChatDirection.INCOMING,
                 Message.resource.isnot(None),
                 Message.correction_id.is_(None),
             )
@@ -890,7 +976,7 @@ class MessageArchiveStorage(AlchemyStorage):
         )
 
         self._explain(session, stmt)
-        return list(session.scalars(stmt))
+        return cast(Sequence[str], session.scalars(stmt).all())
 
     @with_session
     @timeit
@@ -1016,7 +1102,7 @@ class MessageArchiveStorage(AlchemyStorage):
 
             fk_account_pk = self._get_account_pk(session, account)
 
-            now = datetime.now(timezone.utc)
+            now = datetime.now(dt.UTC)
             threshold = now - timedelta(seconds=max_age)
 
             stmt = select(Message).where(
