@@ -5,6 +5,7 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import Literal
 
 import functools
 import inspect
@@ -29,17 +30,11 @@ class OpenEventActionParams(VariantMixin):
     jid: str
 
 
-@dataclass
-class RemoveHistoryActionParams(VariantMixin):
-    account: str
-    jid: JID
-
-
 @dataclass
 class AddChatActionParams(VariantMixin):
     account: str
     jid: JID
-    type: str
+    type: Literal["chat", "groupchat", "pm"]
     select: bool
 
 
@@ -72,6 +67,15 @@ class ModerateMessageParam(VariantMixin):
     namespace: str
 
 
+@dataclass
+class ModerateAllMessagesParam(VariantMixin):
+    account: str
+    jid: JID
+    occupant_id: str
+    nickname: str
+    namespace: str
+
+
 @dataclass
 class DeleteMessageParam(VariantMixin):
     account: str
@@ -82,29 +86,30 @@ class DeleteMessageParam(VariantMixin):
 def get_params_class(func: Callable[..., Any]) -> Any:
     module = sys.modules[__name__]
     params = inspect.signature(func).parameters
-    cls_string = params['params'].annotation
-    cls_string = cls_string.rsplit('.', maxsplit=1)[-1]
+    cls_string = params["params"].annotation
+    cls_string = cls_string.rsplit(".", maxsplit=1)[-1]
     return getattr(module, cls_string)
 
 
-def actionmethod(func: Callable[[Any, Gio.SimpleAction, Any], None]
-                 ) -> Callable[[Any, Gio.SimpleAction, GLib.Variant], None]:
+def actionmethod(
+    func: Callable[[Any, Gio.SimpleAction, Any], None]
+) -> Callable[[Any, Gio.SimpleAction, GLib.Variant], None]:
     @functools.wraps(func)
-    def method_wrapper(obj: Any,
-                       action: Gio.SimpleAction,
-                       param: GLib.Variant) -> None:
+    def method_wrapper(obj: Any, action: Gio.SimpleAction, param: GLib.Variant) -> None:
         params_cls = get_params_class(func)
         params = params_cls.from_variant(param)
         return func(obj, action, params)
+
     return method_wrapper
 
 
-def actionfunction(func: Callable[[Gio.SimpleAction, Any], None]
-                   ) -> Callable[[Gio.SimpleAction, GLib.Variant], None]:
+def actionfunction(
+    func: Callable[[Gio.SimpleAction, Any], None]
+) -> Callable[[Gio.SimpleAction, GLib.Variant], None]:
     @functools.wraps(func)
-    def func_wrapper(action: Gio.SimpleAction,
-                     param: GLib.Variant) -> None:
+    def func_wrapper(action: Gio.SimpleAction, param: GLib.Variant) -> None:
         params_cls = get_params_class(func)
         params = params_cls.from_variant(param)
         return func(action, params)
+
     return func_wrapper
